@@ -2,7 +2,6 @@ from skimage.segmentation import quickshift, mark_boundaries
 import numpy as np
 import matplotlib.pyplot as plt
 from IPython.core.display import display, HTML
-from PIL import Image
 import cv2
 
 
@@ -35,7 +34,7 @@ def show_vis_explanation(explanation_image, cmap=None):
 #         return False  # Probably standard Python interpreter
 
 
-def explanation_to_vis(batched_image, explanation, style='grayscale'):
+def explanation_to_vis(batched_image: np.ndarray, explanation: np.ndarray, style='grayscale') -> np.ndarray:
     """
 
     Args:
@@ -74,20 +73,24 @@ def explanation_to_vis(batched_image, explanation, style='grayscale'):
         raise KeyError("Unknown visualization style.")
 
 
-def _grayscale(explanation, percentile=99):
+def _grayscale(explanation: np.ndarray, percentile=99) -> np.ndarray:
     """
 
     Args:
-        explanation: numpy.array, 2d.
+        explanation: numpy.ndarray, 2d.
         percentile:
 
-    Returns: numpy.array, uint8, same shape as explanation
+    Returns: numpy.ndarray, uint8, same shape as explanation
 
     """
     assert len(explanation.shape) == 2, f"{explanation.shape}. " \
                                         "Currently support 2D explanation results for visualization. " \
                                         "Reduce higher dimensions to 2D for visualization."
 
+    assert np.max(explanation) <= 1.0
+    assert isinstance(percentile, int)
+    assert 0 <= percentile <= 100                       
+
     image_2d = explanation
 
     vmax = np.percentile(image_2d, percentile)
@@ -99,7 +102,7 @@ def _grayscale(explanation, percentile=99):
     return x
 
 
-def overlay_grayscale(image, explanation, percentile=99):
+def overlay_grayscale(image, explanation, percentile=99) -> np.ndarray:
     x = _grayscale(explanation, percentile)
 
     overlay_vis = np.zeros_like(image, dtype=np.uint8)
@@ -110,7 +113,7 @@ def overlay_grayscale(image, explanation, percentile=99):
     return np.uint8(overlay_vis)
 
 
-def _heatmap(explanation, resize_shape=(224, 224)):
+def _heatmap(explanation, resize_shape=(224, 224)) -> np.ndarray:
     """
 
     Args:
@@ -138,7 +141,7 @@ def _heatmap(explanation, resize_shape=(224, 224)):
     return explanation
 
 
-def overlay_heatmap(image, explanation):
+def overlay_heatmap(image, explanation) -> np.ndarray:
     x = _heatmap(explanation, (image.shape[1], image.shape[0]))
 
     overlay_vis = x * 0.4 + image * 0.6
@@ -146,7 +149,7 @@ def overlay_heatmap(image, explanation):
     return np.uint8(overlay_vis)
 
 
-def overlay_threshold(image, explanation_mask):
+def overlay_threshold(image, explanation_mask) -> np.ndarray:
     overlay_vis = np.zeros_like(image, dtype=np.uint8)
     overlay_vis[:, :, 1] = explanation_mask * 255
 
