@@ -1,4 +1,4 @@
-from .abc_interpreter import Interpreter
+from .abc_interpreter import Interpreter, InputGradientInterpreter
 from ..data_processor.readers import preprocess_inputs, preprocess_save_path
 from ..data_processor.visualizer import explanation_to_vis, show_vis_explanation, save_image
 
@@ -6,7 +6,7 @@ import numpy as np
 import paddle
 
 
-class GradShapCVInterpreter(Interpreter):
+class GradShapCVInterpreter(InputGradientInterpreter):
     """
     Gradient SHAP Interpreter for CV tasks.
 
@@ -16,7 +16,8 @@ class GradShapCVInterpreter(Interpreter):
 
     def __init__(self,
                  paddle_model,
-                 use_cuda=True,
+                 use_cuda=None,
+                 device='gpu:0',
                  model_input_shape=[3, 224, 224]) -> None:
         """
         Initialize the GradShapCVInterpreter.
@@ -26,14 +27,8 @@ class GradShapCVInterpreter(Interpreter):
             use_cuda (bool, optional): Whether or not to use cuda. Default: True
             model_input_shape (list, optional): The input shape of the model. Default: [3, 224, 224]
         """
-        Interpreter.__init__(self)
-        self.paddle_model = paddle_model
+        InputGradientInterpreter.__init__(self, paddle_model, device, use_cuda)
         self.model_input_shape = model_input_shape
-        self.paddle_prepared = False
-
-        self.use_cuda = use_cuda
-        if not paddle.is_compiled_with_cuda():
-            self.use_cuda = False
 
     def interpret(self,
                   inputs,
@@ -61,6 +56,15 @@ class GradShapCVInterpreter(Interpreter):
         :rtype: numpy.ndarray
         """
 
+        imgs, data = preprocess_inputs(inputs, self.model_input_shape)
+        bsz = len(data)
+        self.data_type = np.array(data).dtype
+
+        self._build_predict_fn(gradient_of='probability')
+
+        if labels is None:
+            _, labels = self.predict_fn(data, labels)
+
         def add_noise_to_inputs(data):
             max_axis = tuple(np.arange(1, data.ndim))
             stds = noise_amount * (
@@ -71,27 +75,13 @@ class GradShapCVInterpreter(Interpreter):
             ]).astype(self.data_type)
             repeated_data = np.repeat(data, (n_samples, ) * len(data), axis=0)
             return repeated_data + noise
-
-        imgs, data = preprocess_inputs(inputs, self.model_input_shape)
-
-        bsz = len(data)
-        save_path = preprocess_save_path(save_path, bsz)
-
-        self.data_type = np.array(data).dtype
-
+        
         data_with_noise = add_noise_to_inputs(data)
 
         if baselines is None:
             baselines = np.zeros_like(data)
         baselines = np.repeat(baselines, (n_samples, ) * bsz, axis=0)
 
-        if not self.paddle_prepared:
-            self._paddle_prepare()
-
-        if labels is None:
-            _, preds = self.predict_fn(data, labels)
-            labels = preds
-
         labels = np.array(labels).reshape(
             (bsz, 1))  #.repeat(n_samples, axis=0)
         labels = np.repeat(labels, (n_samples, ) * bsz, axis=0)
@@ -107,54 +97,25 @@ class GradShapCVInterpreter(Interpreter):
         gradients, _ = self.predict_fn(input_baseline_points, labels)
 
         input_baseline_diff = data_with_noise - baselines
+        explanations = gradients * input_baseline_diff
 
-        interpretations = gradients * input_baseline_diff
-
-        interpretations = np.concatenate([
+        explanations = np.concatenate([
             np.mean(
-                interpretations[i * n_samples:(i + 1) * n_samples],
+                explanations[i * n_samples:(i + 1) * n_samples],
                 axis=0,
                 keepdims=True) for i in range(bsz)
         ])
 
         # visualization and save image.
+        save_path = preprocess_save_path(save_path, bsz)
         for i in range(bsz):
-            vis_explanation = explanation_to_vis(imgs[i], np.abs(interpretations[i]).sum(0), style='overlay_grayscale')
+            vis_explanation = explanation_to_vis(imgs[i], np.abs(explanations[i]).sum(0), style='overlay_grayscale')
             if visual:
                 show_vis_explanation(vis_explanation)
             if save_path[i] is not None:
                 save_image(save_path[i], vis_explanation)
 
-        return interpretations
-
-    def _paddle_prepare(self, predict_fn=None):
-        if predict_fn is None:
-            paddle.set_device('gpu:0' if self.use_cuda else 'cpu')
-            self.paddle_model.train()
-
-            for n, v in self.paddle_model.named_sublayers():
-                if "batchnorm" in v.__class__.__name__.lower():
-                    v._use_global_stats = True
-                if "dropout" in v.__class__.__name__.lower():
-                    v.p = 0
-
-            def predict_fn(data, labels):
-                data = paddle.to_tensor(data)
-                data.stop_gradient = False
-                out = self.paddle_model(data)
-                out = paddle.nn.functional.softmax(out, axis=1)
-                preds = paddle.argmax(out, axis=1)
-                if labels is None:
-                    labels = preds.numpy()
-                labels_onehot = paddle.nn.functional.one_hot(
-                    paddle.to_tensor(labels), num_classes=out.shape[1])
-                target = paddle.sum(out * labels_onehot, axis=1)
-                target.backward()
-                gradients = data.grad
-                return gradients, labels
-
-        self.predict_fn = predict_fn
-        self.paddle_prepared = True
+        return explanations
 
 
 class GradShapNLPInterpreter(Interpreter):
@@ -174,8 +135,8 @@ class GradShapNLPInterpreter(Interpreter):
             use_cuda (bool, optional): Whether or not to use cuda. Default: True
             model_input_shape (list, optional): The input shape of the model. Default: [3, 224, 224]
         """
-        Interpreter.__init__(self)
-        self.paddle_model = paddle_model
+        Interpreter.__init__(self, paddle_model, 'gpu:0', use_cuda)
+        
         self.use_cuda = use_cuda
         self.paddle_prepared = False
 
